Attribute VB_Name = "modMenu"
Option Explicit
Private MenuList() As TMenu
Private MenuListCount As Long
Private mnuInitialized As Boolean
Private OldProc As Long ' for subclassing

'API Functions
Private Declare Sub CopyMem Lib "kernel32" Alias "RtlMoveMemory" (pDest As Any, pSource As Any, ByVal ByteLen As Long)
Private Declare Function ModifyMenu Lib "user32" Alias "ModifyMenuA" (ByVal hMenu As Long, ByVal nPosition As Long, ByVal wFlags As Long, ByVal wIDNewItem As Long, ByVal lpString As Any) As Long
Private Declare Function GetWindowLong Lib "user32" Alias "GetWindowLongA" (ByVal hwnd As Long, ByVal nIndex As Long) As Long
Private Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hwnd As Long, ByVal nIndex As Long, ByVal dwNewLong As Long) As Long
Private Declare Function CallWindowProc Lib "user32" Alias "CallWindowProcA" (ByVal lpPrevWndFunc As Long, ByVal hwnd As Long, ByVal Msg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Private Declare Function GetDC Lib "user32" (ByVal hwnd As Long) As Long
Private Declare Function ReleaseDC Lib "user32" (ByVal hwnd As Long, ByVal hdc As Long) As Long

Declare Function GetMenuItemID Lib "user32" (ByVal hMenu As Long, ByVal nPos As Long) As Long
Declare Function GetMenu Lib "user32" (ByVal hwnd As Long) As Long
Declare Function GetSubMenu Lib "user32" (ByVal hMenu As Long, ByVal nPos As Long) As Long
Declare Function GetTextExtentPoint32 Lib "gdi32" Alias "GetTextExtentPoint32A" (ByVal hdc As Long, ByVal lpsz As String, ByVal cbString As Long, lpSize As SIZE) As Long

'API Constants
Private Const MF_OWNERDRAW = &H100&
Private Const MF_BYPOSITION = &H400&
Private Const GWL_WNDPROC = (-4)
Private Const WM_DRAWITEM = &H2B
Private Const WM_MEASUREITEM = &H2C
Private Const WM_COMMAND = &H111

'API Structures
Type RECT
        Left As Long
        Top As Long
        Right As Long
        Bottom As Long
End Type

Private Type MEASUREITEMSTRUCT
        CtlType As Long
        CtlID As Long
        itemID As Long
        itemWidth As Long
        itemHeight As Long
        itemData As Long
End Type

Public Type DRAWITEMSTRUCT
        CtlType As Long
        CtlID As Long
        itemID As Long
        itemAction As Long
        itemState As Long
        hwndItem As Long
        hdc As Long
        rcItem As RECT
        itemData As Long
End Type

Type SIZE
        cx As Long
        cy As Long
End Type

Public mnuHeight As Long
Public mnuPicWidth As Long
Public mnuEdgeWidth As Long
Public mnuTextBuffer As Long

Public Sub FreeMenus()
  Dim nIndex As Long
  For nIndex = 0 To MenuListCount
      Set MenuList(nIndex) = Nothing
  Next nIndex
  MenuListCount = 0
  ReDim MenuList(0)
End Sub

Private Sub MeasureItem(ByRef mnu As TMenu, ByRef MIS As MEASUREITEMSTRUCT)
'Menu size
  Dim hDrawDC As Long
  hDrawDC = GetDC(mnu.hwndOwner)
  Dim lpSize As SIZE
  GetTextExtentPoint32 hDrawDC, mnu.Caption, Len(mnu.Caption), lpSize
  If mnuHeight <= 0 Then mnuHeight = 24
  If mnuPicWidth <= 0 Then mnuPicWidth = 24
  MIS.itemHeight = mnuHeight
  MIS.itemWidth = lpSize.cx + mnuPicWidth
  ReleaseDC mnu.hwndOwner, hDrawDC
End Sub
Public Sub MakeOwnerDraw(hMenu As Long, nIndex As Long, nID As Long)
'Modify the menu's attributes
  ModifyMenu hMenu, nIndex, MF_OWNERDRAW Or MF_BYPOSITION, nID, vbNullString
End Sub

Public Function WinProc(ByVal hwnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
'Subclassing proc
  Dim RegID As Long
  If mnuInitialized = False Then
    WinProc = CallWindowProc(OldProc, hwnd, uMsg, wParam, lParam)
    Exit Function
  End If
  Select Case uMsg
     Case WM_DRAWITEM
          Dim DIS As DRAWITEMSTRUCT
          CopyMem DIS, ByVal lParam, Len(DIS)
          For RegID = 0 To MenuListCount
              If (MenuList(RegID).MenuID) = DIS.itemID Then
                 MenuList(RegID).InitDIS DIS.hdc, DIS.itemAction, DIS.itemID, DIS.itemState, DIS.rcItem.Left, DIS.rcItem.Top, DIS.rcItem.Bottom, DIS.rcItem.Right
                 MenuList(RegID).DrawMenu
                 Exit For
              End If
          Next RegID
    Case WM_MEASUREITEM
        Dim MIS As MEASUREITEMSTRUCT
        CopyMem MIS, ByVal lParam, Len(MIS)
        For RegID = 0 To MenuListCount
            If (MenuList(RegID).MenuID) = MIS.itemID Then
                MeasureItem MenuList(RegID), MIS
                Exit For
            End If
        Next RegID
        CopyMem ByVal lParam, MIS, Len(MIS)
    Case Else
        WinProc = CallWindowProc(OldProc, hwnd, uMsg, wParam, lParam)
  End Select
End Function

Public Sub RegisterMenu(hMenu As Long, nPosition As Long, hwndOwner As Long, sCaption As String, objPicture As Object)
  MakeOwnerDraw hMenu, nPosition, GetMenuItemID(hMenu, nPosition)
  If (mnuInitialized = False) Then
     ReDim MenuList(0)
     Set MenuList(0) = New TMenu
     MenuList(0).InitMenu GetMenuItemID(hMenu, nPosition), sCaption, objPicture
     mnuInitialized = True
  Else
     MenuListCount = MenuListCount + 1
     ReDim Preserve MenuList(MenuListCount)
     Set MenuList(MenuListCount) = New TMenu
     MenuList(MenuListCount).hwndOwner = hwndOwner
     MenuList(MenuListCount).InitMenu GetMenuItemID(hMenu, nPosition), sCaption, objPicture
  End If
End Sub

Public Sub SetSubclass(frm As Form)
  OldProc = GetWindowLong(frm.hwnd, GWL_WNDPROC)
  SetWindowLong frm.hwnd, GWL_WNDPROC, AddressOf WinProc
End Sub
